<?php
	
	/**
	 * Dieser Controller übernimmt die Aktionen des Warenkorbs
	 */
	class wpsg_BasketController extends wpsg_SystemController {
		
		private static $_outputCache = [];
		
		public function content_filter(&$content) {

            /**
             * 08.10.2020
             * if (is_singular() && in_the_loop() && is_main_query()) Bedingung ergänzt weil es sonst mit YOAS WP SEO und automatischer
             * Erstellung der Rechnung zu Problemen gekommen ist
             */

			if (wpsg_get_the_id() === $this->shop->getPagePID(wpsg_ShopController::PAGE_BASKET) && (is_singular() && in_the_loop() && is_main_query())) {
				
				parent::dispatch();
				
				if (isset($_REQUEST['wpsg_checkout'])) {
					
					$this->checkoutAction($content);
					
				} else if (isset($_REQUEST['wpsg_checkout2'])) {
					
					$this->checkout2Action($content);
					
				} else if (isset($_REQUEST['wpsg_overview'])) {
					
					$this->overviewAction($content);
					
				} else if (isset($_REQUEST['wpsg_done'])) {
					
					$this->doneAction($content);
					
				} else {
					
					$this->basketAction($content);
					
				}
				
			}
			
		} // public function content_filter($content)

		/**
		 * Gibt den Warenkorb aus, wird von content_filter aufgerufen
		 */
		public function basketAction(&$content) {
			
			if (isset($_REQUEST['wpsg_action']) && $_REQUEST['wpsg_action'] === 'showProdukt' && isset($_REQUEST['produkt_id'])) {
				
				$content = $this->shop->renderProdukt(intval($_REQUEST['produkt_id'])); return;
				
			}
			
			$this->shop->checkCustomerPresetCountry();
			
			// Basket aus Session zusammenbauen
			$this->shop->basket->initFromSession(true);
			$this->shop->basket->save(false);
			
			// Basket muss aufgebaut sein
			$this->shop->checkShippingAvailable();
			$this->shop->checkPaymentAvailable();            
			$this->shop->checkCustomerPreset();
			
			\wpsg\wpsg_calculation::getSessionCalculation()->update();
			
			$this->shop->basket->oCalculation = null;
			$this->shop->view['basket'] = $this->shop->basket->toArray(false, false);

			$this->shop->view['error'] = array();
			if (isset($_SESSION['wpsg']['errorFields']) && wpsg_checkInput($_SESSION['wpsg']['errorFields'], WPSG_SANITIZE_ARRAY_TEXTFIELD)) $this->shop->view['error'] = $_SESSION['wpsg']['errorFields'];
 			
			$this->shop->view['arLander'] = $this->db->fetchAssocField("SELECT L.`id`, L.`name` FROM `".WPSG_TBL_LAND."` AS L ORDER BY L.`name` ", "id", "name");
			
			// colspan für die Zusammenfassung berechnen je nach Option
			$this->shop->view['colspan'] = 3;
			if ($this->get_option('wpsg_showMwstAlways') == '1' || sizeof($this->shop->view['basket']['mwst']) > 1) $this->shop->view['colspan'] ++;

			if ($this->shop->hasMod('wpsg_mod_onepagecheckout') && ($this->get_option('wpsg_mod_onepagecheckout_basket') == 1)) {
				
				$this->shop->view['arShipping'] = $this->shop->arShipping;
				$this->shop->view['arPayment'] = $this->shop->arPayment;
				$this->shop->view['pflicht'] = $this->shop->loadPflichtFeldDaten();
				$this->shop->view['laender'] = $this->db->fetchAssoc("SELECT * FROM `".WPSG_TBL_LAND."` ORDER BY `name` ASC");
				
				$content = $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_onepagecheckout/onepage.phtml', false);
				
			} else {
				
				$content = $this->shop->render(WPSG_PATH_VIEW.'/warenkorb/basket.phtml', false);
				
			}
			
		} // public function basketAction()
		
		public function setBasketData() {
			
			// gewählte Lieferanschrift im Warenkorb setzen
			if (isset($_REQUEST['set_land'])) {
			
				$set_land = $_REQUEST['set_land'];
				
				if ($this->shop->hasMod('wpsg_mod_shippingadress') && wpsg_isSizedInt($_SESSION['wpsg']['checkout']['shipping_land'])) {
					
					$_SESSION['wpsg']['checkout']['shipping_land'] = $set_land;
					
				} else {
					
					$_SESSION['wpsg']['checkout']['land'] = $set_land;
					
				}
				
				// Für die Abfrage von EU Leistungsort Produkten eintragen
				if (wpsg_isSizedInt($_SESSION['wpsg']['customerCountry'])) $_SESSION['wpsg']['customerCountry'] = $set_land;
				
			}
			
			// gewählte Versandart setzen
			if (isset($_REQUEST['set_shipping']) && wpsg_checkInput($_REQUEST['set_shipping'], WPSG_SANITIZE_SHIPPING_KEY)) {
				
				$_SESSION['wpsg']['checkout']['shipping'] = $_REQUEST['set_shipping'];
				
			}
			
			// gewählte Zahlungsart setzen
			if (isset($_REQUEST['set_payment']) && wpsg_checkInput($_REQUEST['set_payment'], WPSG_SANITIZE_PAYMENT_KEY)) {
				
				$_SESSION['wpsg']['checkout']['payment'] = $_REQUEST['set_payment'];
				
			}
			
			$this->shop->callMods('setBasketData');
			
		} // public function setBasketData()
		
		/**
		 * Eingabe der Kundendaten (Checkout)
		 * @param $content
		 * @throws Exception
		 */
		public function checkoutAction(&$content) {

			$this->setBasketData();
			
			$this->shop->checkCustomerPresetCountry();
			
			// Basket aus Session zusammenbauen
			$this->shop->basket->initFromSession(true);
			$this->shop->basket->save(false);
			
			// Basket muss aufgebaut sein
			$this->shop->checkShippingAvailable();
			$this->shop->checkPaymentAvailable();            
			$this->shop->checkCustomerPreset();
			
			\wpsg\wpsg_calculation::getSessionCalculation()->update();
			
			$bError = true;
			$this->shop->callMods('checkBasket', array(&$bError));
			
			if ($bError === false) {
				
				$this->basketAction($content);
				
				return;
				
			}
			
			// Wenn keine Produkte drin sind, dann mit Fehlermeldung zum Warenkorb leiten
			$arProductIDs = $this->shop->basket->getProductIDs();
			
			if (!wpsg_checkInput($arProductIDs, WPSG_SANITIZE_ARRAY_INT)) {
				
				$this->shop->addFrontendError(__('Es befinden sich keine Produkte im Warenkorb.', 'wpsg'));
				$this->basketAction($content);
				
				return;
				
			}
			 			
			if ($this->shop->hasMod('wpsg_mod_onepagecheckout')) {
				
				if ($this->shop->get_option('wpsg_mod_onepagecheckout_basket') == 1) {
					
					$this->basketAction($content); return;
					
				} else {
					
					$content = $this->shop->callMod('wpsg_mod_onepagecheckout', 'onepage'); 
					
					return;
					
				}
				
			}
			
			$this->shop->view['pflicht'] = $this->shop->loadPflichtFeldDaten();
			
			$this->shop->view['error'] = array();
			if (wpsg_checkInput($_SESSION['wpsg']['errorFields'], WPSG_SANITIZE_ARRAY_TEXTFIELD)) $this->shop->view['error'] = $_SESSION['wpsg']['errorFields'];
			
			$this->shop->view['basket'] = $this->shop->basket->toArray();
			$this->shop->view['basket']['checkout']['geb'] = wpsg_fromDate($this->shop->view['basket']['checkout']['geb'], true);
			
			$this->shop->view['laender'] = $this->db->fetchAssoc("SELECT * FROM `".WPSG_TBL_LAND."` ORDER BY `name` ASC");
			
			$this->shop->view['arAnrede']= explode('|', $this->shop->get_option('wpsg_admin_pflicht')['anrede_auswahl']);
			
			$content = $this->shop->render(WPSG_PATH_VIEW.'/warenkorb/checkout.phtml', false);
			
		} // public function checkoutAction()
		
		/**
		 * Eingabe der Versand- und Zahlungsart
		 */
		private function checkout2Action(&$content) {

			$this->shop->basket->initFromSession();

			// Wenn keine Produkte drin sind, dann mit Fehlermeldung zum Warenkorb leiten
			$temp = $this->shop->basket->getProductIDs();
			
			if (!wpsg_checkInput($temp, WPSG_SANITIZE_ARRAY_INT)) {
				
				$this->shop->addFrontendError(__('Es befinden sich keine Produkte im Warenkorb.', 'wpsg'));
				$this->basketAction($content);
				
				return;
				
			}
			
			$this->shop->view['error'] = array();
			if (wpsg_checkInput($_SESSION['wpsg']['errorFields'], WPSG_SANITIZE_ARRAY_TEXTFIELD)) $this->shop->view['error'] = $_SESSION['wpsg']['errorFields'];
			
			$this->shop->view['basket'] = $this->shop->basket->toArray();
			 
			$this->shop->checkShippingAvailable();
			$this->shop->checkPaymentAvailable();
			
			if (sizeof($this->shop->arShipping) > 0 && (!in_array($_SESSION['wpsg']['checkout']['shipping'], array_keys($this->shop->arShipping)) || !in_array($_SESSION['wpsg']['checkout']['payment'], array_keys($this->shop->arPayment)))) {
				
				if (!in_array($_SESSION['wpsg']['checkout']['shipping'], array_keys($this->shop->arShipping))) $_SESSION['wpsg']['checkout']['shipping'] = array_keys($this->shop->arShipping)[0];
				if (!in_array($_SESSION['wpsg']['checkout']['payment'], array_keys($this->shop->arPayment))) $_SESSION['wpsg']['checkout']['payment'] = array_keys($this->shop->arPayment)[0];
				
				$this->shop->basket->initFromSession(true);
				$this->shop->view['basket'] = $this->shop->basket->toArray();
				
			}
			
			$this->shop->view['arShipping'] = $this->shop->arShipping;
			$this->shop->view['arPayment'] = $this->shop->arPayment;
			
			$content = $this->shop->render(WPSG_PATH_VIEW.'/warenkorb/checkout2.phtml', false);
			
		} // private function checkout2Action()
		
		/**
		 * Funktion für die Bestellzusammenfassung
		 */
		private function overviewAction(&$content) {

			if (!isset($this->_outputCache['overview'])) {
				
				$this->shop->checkCustomerPresetCountry();
				
				$this->shop->basket->initFromSession();
				
				// Basket muss aufgebaut sein
				$this->shop->checkShippingAvailable();
				$this->shop->checkPaymentAvailable();            
				$this->shop->checkCustomerPreset();
				
				\wpsg\wpsg_calculation::getSessionCalculation()->update();
				
				// Wenn keine Produkte drin sind, dann mit Fehlermeldung zum Warenkorb leiten
				$temp = $this->shop->basket->getProductIDs();

				if (!wpsg_checkInput($temp, WPSG_SANITIZE_ARRAY_INT)) {
					
					$this->shop->addFrontendError(__('Es befinden sich keine Produkte im Warenkorb.', 'wpsg'));
					$this->basketAction($content);
					
					return;
					
				}

				// Wenn die Overview Prüfung nicht bestanden wurde, dann zum Checkout leiten
				if (!$this->shop->basket->checkCheckout(3)) {
					
					if ($this->shop->hasMod('wpsg_mod_onepagecheckout') && ($this->shop->get_option('wpsg_mod_onepagecheckout_basket') == 1)) {
						
						$this->basketAction($content);
						
						return;
					
					} else {
						
						$this->checkoutAction($content);
						
						return;
						
					}
					
				}
		 
				$this->shop->view['pflicht'] = $this->shop->loadPflichtFeldDaten();
				
				$this->shop->view['error'] = array();
				if (wpsg_checkInput($_SESSION['wpsg']['errorFields'], WPSG_SANITIZE_ARRAY_TEXTFIELD)) $this->shop->view['error'] = $_SESSION['wpsg']['errorFields'];			

				$this->shop->view['basket'] = $this->shop->basket->toArray();
								
				if (!isset($this->shop->arShipping[$this->shop->view['basket']['checkout']['shipping']]) || !isset($this->shop->arShipping[$this->shop->view['basket']['checkout']['payment']])) {
					
					$this->shop->basket->initFromSession(true);
					$this->shop->view['basket'] = $this->shop->basket->toArray();
					
				}

            	$this->shop->view['arShipping'] = $this->shop->arShipping;
            	$this->shop->view['arPayment'] = $this->shop->arPayment;
            
				// colspan für die Zusammenfassung berechnen je nach Option
				$this->shop->view['colspan'] = 2;
				if ($this->get_option('wpsg_showMwstAlways') == '1' || sizeof($this->shop->view['basket']['mwst']) > 1) $this->shop->view['colspan'] ++;

				self::$_outputCache['overview'] = $this->shop->render(WPSG_PATH_VIEW.'/warenkorb/overview.phtml', false);
	
			 }
			
			$content = self::$_outputCache['overview'];
			
		} // private function overviewAction()
		
		/**
		 * Seite, die nach der Bestellung angezeigt wird
		 * @param $content
		 * @throws \wpsg\Exception
		 * @throws Exception
		 */
		private function doneAction(&$content) {
			
			if (!wpsg_checkInput($_REQUEST['order_id'], WPSG_SANITIZE_INT)) throw \wpsg\Exception::getSanitizeException();
			else {
								
				// Hash verifizierne
				$order_id = intval($_REQUEST['order_id']);
				
				if ($order_id <= 0) throw \wpsg\Exception::getInvalidValueException();
							
				$oOrder = wpsg_order::getInstance($order_id);
				
				$hash = md5($oOrder->__get('secret').$order_id.$oOrder->__get('secret'));
				
				if (hash_equals($hash, rawurldecode($_REQUEST['wpsg_done']))) {					
					 
					$this->shop->basket->initFromDB($order_id);
					$this->shop->view['basket'] = $this->shop->basket->toArray();
					$this->shop->view['o_id'] = $order_id;
					$this->shop->view['order'] = $this->shop->cache->loadOrder($order_id);
					$this->shop->view['customer'] = $this->shop->cache->loadKunden($this->shop->view['order']['k_id']);
					
					$content = $this->shop->render(WPSG_PATH_VIEW.'/warenkorb/done.phtml', false);	
					
				} else $this->addFrontendError(__('Der Link ist ausgelaufen oder ungültig.', 'wpsg'));
				
			}
				
		} // private function doneAction()
		
		/**
		 * Verarbeitet die Anfragen und leitet anschließend weiter
		 */
		public function template_redirect() {

			if (isset($_REQUEST['wpsg_cron'])) {
				
				// ersetzt den direkten Aufruf der cron.php
				
				$this->shop->callMods('cron');
				$this->shop->update_option('wpsg_lastCron', time());

				exit;
				
			}			
			
			if (isset($_REQUEST['wpsg_form_data'])) {

				parse_str($_REQUEST['wpsg_form_data'], $request);
				$_REQUEST = array_merge_recursive($_REQUEST, $request);
				
			}
			
			if (isset($_REQUEST['wpsg']['action']) && $_REQUEST['wpsg']['action'] === 'customeranswer') {
				
				// Der Preisdialog wurde beantwortet
				$_SESSION['wpsg']['priceDialog'] = true;
				$_SESSION['wpsg']['customertype'] = intval($_REQUEST['wpsg']['customertype']);
				$_SESSION['wpsg']['checkout']['land'] = intval($_REQUEST['wpsg']['customerCountry']);
				
				if (!wpsg_checkInput($_REQUEST['wpsg']['redirect'], WPSG_SANITIZE_URL_LOCAL)) throw \wpsg\Exception::getSanitizeException();
				
				$this->redirect($_REQUEST['wpsg']['redirect']);
				
			} else if (wpsg_isSizedString($_REQUEST['wpsg']['action'], 'updateCheckout')) {
				
				// Checkout Aktualisierung über AJAX				
				if (isset($_REQUEST['wpsg']['checkout']['shipping']) && wpsg_checkInput($_REQUEST['wpsg']['checkout']['shipping'], WPSG_SANITIZE_SHIPPING_KEY)) $_SESSION['wpsg']['checkout']['shipping'] = $_REQUEST['wpsg']['checkout']['shipping'];
				if (isset($_REQUEST['wpsg']['checkout']['payment']) && wpsg_checkInput($_REQUEST['wpsg']['checkout']['payment'], WPSG_SANITIZE_PAYMENT_KEY)) $_SESSION['wpsg']['checkout']['payment'] = $_REQUEST['wpsg']['checkout']['payment'];
				
				die("1");
				
			} else if (wpsg_isSizedString($_REQUEST['wpsg']['action'], 'widget')) {
				
				the_widget('wpsg_basket_widget');
				
				die();
				
			}
			 
			// Der Warenkorb wurde abgeschickt
			if (isset($_REQUEST['wpsg_basket_submit'])) {
				
				// Ein Modul hat einen Fehler im Warenkorb entdeckt, zurück zum Warenkorb leiten
				if ($this->shop->callMods('basket_checkoutAction', array(&$this)) == false) {
					
					$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
					
				}
				
			}
			
			// Warenkorb wurde verarbeitet, soll ein Modul die Umleitung übernehmen?
			if (wpsg_isSizedString($_REQUEST['wpsg_mod_submit'])) {
				
				$this->shop->callMod($_REQUEST['wpsg_mod_submit'], 'basket_submitSuccess');
				
			}
			
			if (isset($_REQUEST['wpsg']['action']) && $_REQUEST['wpsg']['action'] === 'customerquestion') {
				
				// Das Widget mit der Frage bzgl. der Preisberechnung EU Rotz soll angezeigt werden
				parent::dispatch();
				
				$this->shop->view['defaultCountry_id'] = $this->shop->getFrontendCountry(true);
				
				$this->shop->view['url'] = '';
				if (isset($_REQUEST['wpsg']['url']) && wpsg_checkInput($_REQUEST['wpsg']['url'], WPSG_SANITIZE_URL_LOCAL)) $this->shop->view['url'] = $_REQUEST['wpsg']['url'];
				
				$geo_code = wpsg_geo_code();
				$country_id = $this->db->fetchOne("SELECT L.`id` FROM `".WPSG_TBL_LAND."` AS L WHERE L.`kuerzel` = '".wpsg_q($geo_code)."' ");
				if (wpsg_isSizedInt($country_id)) $this->shop->view['defaultCountry_id'] = $country_id;
				
				$this->shop->view['arCountry'] = wpsg_country::find();
				
				die($this->shop->render(WPSG_PATH_VIEW.'/warenkorb/customerquestion.phtml'));
				
			} else if (isset($_REQUEST['wpsg']['submit']) && intval($_REQUEST['wpsg']['menge']) > 0 && intval($_REQUEST['wpsg']['produkt_id']) > 0) {

				// Ein Produktformular wurde abgeschickt
				// http://wp.home/wpsg4/warenkorb/?wpsg[produkt_id]=6&wpsg[submit]=1&wpsg[menge]=1
				// https://shop.maennchen1.de/warenkorb/?wpsg[produkt_id]=63&wpsg[submit]=1&wpsg[menge]=1&wpsg_vp[5]=10
				
				parent::dispatch();
				
				$this->shop->callMods('basket_preInsert');
				
				$product_key = intval($_REQUEST['wpsg']['produkt_id']);
				$this->shop->callMods('getProductKeyFromRequest', array(&$product_key, $_REQUEST['wpsg']['produkt_id'], $_REQUEST));
				
				// Produkt hinzufügen
				$amount = intval($_REQUEST['wpsg']['menge']);
				$bOK = $this->shop->basket->addProduktToSession($product_key, $amount);

				if ($this->shop->get_option('wpsg_afterinsert') == '2') die();
				
				if ($this->shop->get_option('wpsg_afterinsert') == '3') {
					
					$this->shop->basket->initFromSession(true);
					
					$this->shop->view['product_data'] = $this->shop->loadProduktArray($this->shop->getProduktId($product_key));
					$this->shop->view['oProduct'] = wpsg_product::getInstance($this->shop->getProduktID($product_key));
					$this->shop->view['product_key'] = $product_key;
					$this->shop->view['amount_add'] = $amount;
					$this->shop->view['amount_basket'] = $this->shop->basket->getBasketAmount($product_key);
					$this->shop->view['product_index'] = $GLOBALS['wpsg_lastInsertIndex'];
					
				}
				
				if ($this->shop->get_option('wpsg_afterinsert') == '4') {
					
					if (wpsg_checkInput($_REQUEST['myReferer'], WPSG_SANITIZE_URL_LOCAL)) $this->redirect($_REQUEST['myReferer']);
					else throw \wpsg\Exception::getSanitizeException();
					
				}
				
				// Erfolgsmeldung hinzufügen
				if ($bOK === true) $this->shop->addFrontendMessage(__('Produkt erfolgreich in den Warenkorb gelegt.', 'wpsg'));
				else if ($bOK === -1) {
					
					// Hier Warenkorb ohne Meldung anzeigen / Für den Fall, dass das Produkt nur einmal im Warenkorb auftauchen darf
					
				} else {
					
					if ($this->shop->get_option('wpsg_afterinsert') == '3') {
						
						echo $this->shop->render(WPSG_PATH_VIEW.'warenkorb/messageDialog.phtml');
						
						exit;
						
					}
					
					// Fehler beim hinzufügen, hier einfach auf der Seite bleiben					
					
					if (wpsg_checkInput($_REQUEST['myReferer'], WPSG_SANITIZE_URL_LOCAL)) $this->redirect($_REQUEST['myReferer']);
					else \wpsg\Exception::getSanitizeException();
					
				}
				
				if (wpsg_isSizedInt($_REQUEST['wpsg']['ajax'], 1)) {
					
					if ($this->shop->get_option('wpsg_afterinsert') == '3') {
						
						$this->shop->view['product_data']['preis'] = $this->shop->view['oProduct']->getPrice($product_key, $this->shop->getFrontendTaxview());						
						$this->shop->view['content'] = $this->shop->render(WPSG_PATH_VIEW.'warenkorb/ajaxDialog.phtml', false);
						
						echo $this->shop->render(WPSG_PATH_VIEW.'warenkorb/messageDialog.phtml');
						
						exit;
						
					} else {
						
						echo "1";
						
						exit;
						
					}
					
				} else if (wpsg_isSizedInt($this->get_option('wpsg_afterinsert'), 1)) {
					
					$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
					
				} else {
					
					if (wpsg_checkInput($_REQUEST['myReferer'], WPSG_SANITIZE_URL_LOCAL)) $this->redirect($_REQUEST['myReferer']);
					else throw \wpsg\Exception::getSanitizeException();
					
				}
				
			} else if (isset($_REQUEST['wpsg_basket_ajax'])) {
				
				$content = '';
				
				$this->basketAction($content);
				
				echo $content;
				
				exit;
				
			} else if (isset($_REQUEST['wpsg_redirect_basket'])) {
				
				// Button "Zurück zum Warenkorb wurde gedrückt"
				// -> Abbruch und Checkout nicht speichern
				
				$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
				
			} else if (isset($_REQUEST['wpsg_redirect_checkout'])) {
				
				// Weiterleitung zum Checkout ohne speichern
				// Wird normalerweise bei "Zurück zur Kasse" im Checkout2 aufgerufen
				
				$this->redirect($this->shop->getURL(wpsg_ShopController::URL_CHECKOUT));
				
			} else if (isset($_REQUEST['wpsg_gutschein_add'])) {
				
				if (!isset($_SESSION['wpsg']['errorFields'])) $_SESSION['wpsg']['errorFields'] = [];
				
				// Gutscheincode soll hinzugefügt werden
				$code = sanitize_text_field($_REQUEST['wpsg']['gutschein']);
				
				$gs_db = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_GUTSCHEIN."` WHERE `code` = '".wpsg_q($code)."' ");
				
				if (!isset($gs_db['id']) || $gs_db['id'] <= 0) {
					
					$this->addFrontendError(__('Ein Gutschein mit diesem Code existiert nicht!', 'wpsg'));
					
					$_SESSION['wpsg']['errorFields'][] = 'gutschein';
					
					$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
					
				} else {
					
					// Gültigkeit des Gutscheins prüfen
					$tStart = strtotime($gs_db['start_date']);
					$tEnd = strtotime($gs_db['end_date']);
					
					$arBasket = $this->shop->cache->getShopBasketArray();
					
					if (!(wpsg_time() >= $tStart && wpsg_time() <= $tEnd)) {
						
						// Zeit ist abgelaufen	
						$this->addFrontendError(__('Dieser Gutscheincode ist nicht gültig!', 'wpsg'));
						$_SESSION['wpsg']['errorFields'][] = 'gutschein';
						
						$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
						
					} else {
						
						if ($gs_db['o_id'] > 0 && $gs_db['multi'] !== '1') {
							
							// Gutschein ist bereits verbraucht
							$this->addFrontendError(__('Dieser Gutscheincode wurde bereits eingelöst und ist nicht mehrfach verwendbar.', 'wpsg'));
							$_SESSION['wpsg']['errorFields'][] = 'gutschein';
							
						} else {
							
							$bCheckDiscount = true;
							
							if ($this->shop->get_option('wpsg_mod_discount_voucher') === '1') {
								
								if ($this->shop->callMod('wpsg_mod_discount', 'hasDiscount', [$arBasket]) === true) $bDiscount = true;
								else $bDiscount = false;
								
								if ($bDiscount) {
									
									$this->addFrontendError(__('Dieser Gutscheincode kann nicht eingelöst werden, da bereits Rabatte angerechnet sind.', 'wpsg'));
									
									$_SESSION['wpsg']['errorFields'][] = 'gutschein';
									
									$bCheckDiscount = false;
									
								}
								
							}

							if ($bCheckDiscount) {
								
								$oVoucher = wpsg_voucher::getInstance(intval($gs_db['id']));
								
								if (!$oVoucher->isUsabel()) {
									
									$this->addFrontendError(__('Gutschein kann nicht verwendet werden. Er ist nicht mehr aktiv oder verbraucht.', 'wpsg'));
									
								} else {
									
									// Gutschein ist von der größe des Warenkorbes begrenzt.
									if (wpsg_tf($gs_db['minvalue']) > 0 && wpsg_tf($gs_db['minvalue']) > $arBasket['sum']['preis']) {
										
										$this->addFrontendError(wpsg_translate(__('Dieser Gutscheincode kann nicht eingelöst werden, da der Mindestbestellwert von #1# noch nicht erreicht ist.', 'wpsg'), wpsg_ff(wpsg_tf($gs_db['minvalue']), $this->shop->get_option('wpsg_currency'))));
										
										$_SESSION['wpsg']['errorFields'][] = 'gutschein';
										
									} else {
										
										$this->shop->basket->initFromSession();
										$arProductIDsBasket = $this->shop->basket->getProductIDs();
										
										$wrongProducts = array();
										
										// Produktgültigkeit prüfen
										/*
										$tarr = wpsg_explode(',', $gs_db['products']);
										if (wpsg_isSizedArray($tarr)) {
											
											if (sizeof(array_diff($arProductIDsBasket, wpsg_explode(',', $gs_db['products']))) > 0) {
												
												$wrongProducts = array_diff($arProductIDsBasket, wpsg_explode(',', $gs_db['products']));
												
											}
											
										}
										
										// Produktgruppengültigkeit prüfen
										$tarr = wpsg_explode(',', $gs_db['productgroups']);
										if (wpsg_isSizedArray($tarr)) {
											
											$ProductGroupIDs = wpsg_explode(',', $gs_db['productgroups']);
											
											$arProductIDs = $this->db->fetchAssocField("
												SELECT P.`id` FROM `".WPSG_TBL_PRODUCTS."` AS P WHERE P.`pgruppe` IN (".wpsg_implode(',', $ProductGroupIDs).")
											");
											
											if (sizeof(array_diff($arProductIDsBasket, $arProductIDs)) > 0) {
												
												$wrongProducts = array_merge($wrongProducts, array_diff($arProductIDsBasket, $arProductIDs));
												
											}
											
										}
										
										if (wpsg_isSizedArray($wrongProducts)) {
											
											if (sizeof($wrongProducts) === 1) {
												
												$product = $this->shop->loadProduktArray(array_values($wrongProducts)[0]);
												$this->addFrontendError(wpsg_translate(__('Gutschein konnte nicht hinzugefügt werden, da er für das Produkt #1# nicht zulässig ist.', 'wpsg'), $product['name']));
												
											} else {
												
												$arProductNames = array();
												
												foreach ($wrongProducts as $product_id) {
													
													$product_data = $this->shop->loadProduktArray($product_id);
													$arProductNames[] = $product_data['name'];
													
												}
												
												$this->addFrontendError(wpsg_translate(__('Gutschein konnte nicht hinzugefügt werden, da er für die Produkte #1# nicht zulässig ist.', 'wpsg'), wpsg_implode(', ', $arProductNames)));
												
											}
											
										} else {
											
											$this->addFrontendMessage(__('Gutschein wurde der Bestellung erfolgreich hinzugefügt.', 'wpsg'));
											$this->shop->basket->addGutscheinToSession($gs_db['value'], $gs_db['calc_typ'], $gs_db['code'], $gs_db['id']);
											
										}
										*/

										$this->addFrontendMessage(__('Gutschein wurde der Bestellung erfolgreich hinzugefügt.', 'wpsg'));
										$this->shop->basket->addGutscheinToSession($gs_db['value'], $gs_db['calc_typ'], $gs_db['code'], $gs_db['id']);
										
									}
									
								}
								
							}
							
						}
						
						$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
						
					}
					
				}
				
			} else if (isset($_REQUEST['wpsg_order'])) {
				
				parent::dispatch();
								
				$this->shop->checkCustomerPresetCountry();
				
				// Bestellung abschließen
				$this->shop->basket->initFromSession(true);
				
				// Basket muss aufgebaut sein
				$this->shop->checkShippingAvailable();
				$this->shop->checkPaymentAvailable();            
				$this->shop->checkCustomerPreset();
				 
				\wpsg\wpsg_calculation::getSessionCalculation()->update();
				
				$bOK = $this->shop->basket->checkCheckout();
				
				if ($bOK) {
					
					$bModulError = false; $this->shop->callMods('checkFinaly', array(&$bModulError));
					
					if ($bModulError === true) {
						
						$this->redirect($this->shop->getURL(wpsg_ShopController::URL_OVERVIEW));
						
					} else if (!$this->shop->hasMod('wpsg_mod_ordercondition') && wpsg_getStr($_REQUEST['wpsg']['agb']) != '1' ) {
						
						$this->addFrontendError(
							wpsg_translate(
								__('Sie müssen unsere #1# und #2# gelesen haben, um eine Bestellung durchzuführen! Bitte setzen Sie unten das entsprechende Häkchen!', 'wpsg'),
								'<a href="'.$this->shop->getURL(wpsg_ShopController::URL_AGB).'">'.__('AGB', 'wpsg').'</a>',
								'<a href="'.$this->shop->getURL(wpsg_ShopController::URL_WIDERRUF).'">'.__('Widerrufsbelehrung', 'wpsg').'</a>'
							)
						);
						
						$this->redirect($this->shop->getURL(wpsg_ShopController::URL_OVERVIEW));
						
					} else { 
						
						$order_id = $this->shop->basket->save();
						
						$this->redirect($this->shop->getDoneURL($order_id));
						
					}
					
				} else {
					
					$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
					
				}
				
			} else if (wpsg_isSizedString($_REQUEST['wpsg_action'], 'remove') && isset($_REQUEST['wpsg_produkt'])) {
				
				parent::dispatch();
				
				if (preg_match('/^voucher_(\d+)?/', $_REQUEST['wpsg_produkt'], $m)) {
					
					foreach ($_SESSION['wpsg']['gs'] as $gs_k => $gs) {
						
						if (intval($gs['id']) === intval($m[1])) unset($_SESSION['wpsg']['gs'][$gs_k]);
						
					}
					
					$this->shop->addFrontendMessage(__('Gutschein erfolgreich entfernt.', 'wpsg'));
					
				} else {
					
					// Produkt entfernen
					if ($this->shop->basket->removeProduktFromSession(intval($_REQUEST['wpsg_produkt']))) {
						
						// Erfolgsmeldung hinzufügen
						$this->shop->addFrontendMessage(__('Produkt erfolgreich entfernt.', 'wpsg'));
						
					}
					
					$this->shop->callMods('basket_afterRemove');
					
				}
				
				$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
				
			} else if (isset($_REQUEST['wpsg_basket_refresh'])) {
				
				parent::dispatch();
				
				$this->shop->callMods('basket_preUpdate');
				
				$bError = false;
				
				// Warenkorb aktualisieren

				foreach ($_REQUEST['wpsg']['menge'] as $product_index => $produkt_menge) {
					
					$product_key = $_SESSION['wpsg']['basket'][$product_index]['id'];
					
					if (!wpsg_checkInput($product_key, WPSG_SANITIZE_PRODUCTKEY)) \wpsg\Exception::getSanitizeException();
					
					$product_data = $this->shop->cache->loadProduct($this->shop->getProduktId($product_key));
					
					if ($product_data['basket_multiple'] == 1) {
						
						// Mehrfach mit beliebiger Menge
						if (!$this->shop->basket->updateProduktFromSession($product_index, $produkt_menge)) $bError = true;
						
					} else if ($product_data['basket_multiple'] == 2) {
						
						// Mehrfach mit Menge 1
						if ($produkt_menge > 1) {
							
							// Grundprodukt auf 1 setzen
							$this->shop->basket->updateProduktFromSession($product_index, 1);
							
							$request = $_REQUEST;
							$this->shop->callMods('basket_preMultiple', array($product_index));
							
							$bOK = true; for ($i = 1; $i < $produkt_menge; $i ++) {
								
								$bOK = $bOK && $this->shop->basket->addProduktToSession($product_key, 1);
								
							}
							
							$_REQUEST = $request;
							
							if (!$bOK) $bError = true;
							
						} else {
							
							// Menge ist 1 gebliegen, hier normal aktualisieren
							if (!$this->shop->basket->updateProduktFromSession($product_index, $produkt_menge)) $bError = true;
							
						}
						
					} else if ($product_data['basket_multiple'] == 4) {
						
						if (intval($produkt_menge) !== 1) {
							
							$this->shop->addFrontendError(__('Produkt darf nur einmal im Warenkorb auftauchen.', 'wpsg'));
							
						}
						
					} else {
						
						// Nur einmal mit beliebiger Menge
						if (!$this->shop->basket->updateProduktFromSession($product_index, $produkt_menge)) $bError = true;
						
					}
					
				} // foreach
				
				$this->setBasketData();
				$this->shop->cache->clearShopBasketArray();
				
				$this->shop->callMods('basket_afterUpdate', array(&$bError));
				
				if (!$bError) {
					
					// Fehler ist aufgetreten
					$this->shop->addFrontendMessage(__('Warenkorb erfolgreich aktualisiert', 'wpsg'));
					
				}
				
				$this->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));
				
			} else if (wpsg_isSizedArray($_REQUEST['wpsg']['checkout']) && !isset($_REQUEST['wpsg_mod'])) {

				// Der Ausschluß von wpsg_mod ist nötig, da PayPal PLUS die Versandart ändert

                if (wpsg_isSizedString($this->shop->basket->arCheckout['email']) && $this->shop->get_option('wpsg_save_order_after_mail') === '1') {

                    // Speichern in der Datenbank
                    $this->shop->basket->save(false, false, true);

                }

				// Checkout wurde abgeschickt
				parent::dispatch();
				
				if (wpsg_isSizedInt($_SESSION['wpsg']['customerCountry']) && $_SESSION['wpsg']['customerCountry'] != $_SESSION['wpsg']['land']) $_SESSION['wpsg']['customerCountry'] = intval($_SESSION['wpsg']['land']);
				
				$this->shop->callMods('doCheckout');				
				$this->shop->basket->initFromSession(true);
				
				// Checkout1 muss immer geprüft werden, ist z.B. für das Kundenmodul notwendig!				
				$bOK = $this->shop->basket->checkCheckout(1);
				
				if (isset($_REQUEST['wpsg_checkout'])) {
					
					// Änderungen bei der Lieferadresse nach dem "overview" beachten
					if (isset($_REQUEST['wpsg']['checkout']['act_checkout_shippingadress'])) {
						
						if ($_REQUEST['wpsg']['checkout']['act_checkout_shippingadress'] !== 'act') unset($_SESSION['wpsg']['checkout']['act_checkout_shippingadress']);
						
					}
					
					if ($this->shop->hasMod('wpsg_mod_onepagecheckout')) {
						
						$bOK = $bOK && $this->shop->basket->checkCheckout(2);
						
						$target_error = $this->shop->getURL(wpsg_ShopController::URL_CHECKOUT);
						$target = $this->shop->getURL(wpsg_ShopController::URL_OVERVIEW);
						
					} else {
						
						$target_error = $this->shop->getURL(wpsg_ShopController::URL_CHECKOUT);
						
						$this->shop->checkShippingAvailable();
						$this->shop->checkPaymentAvailable();            
						$this->shop->checkCustomerPreset();
						
						if (array_key_exists($this->shop->basket->arCheckout['shipping'], $this->shop->arShipping) && array_key_exists($this->shop->basket->arCheckout['payment'], $this->shop->arPayment) && $this->shop->get_option('wpsg_skip_checkout2') === '1' && $this->shop->basket->arCheckout['payment'] != 20) {
							
							$target = $this->shop->getURL(wpsg_ShopController::URL_OVERVIEW);
							
						} else {
							
							$target = $this->shop->getURL(wpsg_ShopController::URL_CHECKOUT2);
							
						}
						
					}
					
				} else if (isset($_REQUEST['wpsg_checkout2'])) {
					
					$this->shop->checkCustomerPresetCountry();
				
					// Bestellung abschließen
					$this->shop->basket->initFromSession(true);
					
					// Basket muss aufgebaut sein
					$this->shop->checkShippingAvailable();
					$this->shop->checkPaymentAvailable();            
					$this->shop->checkCustomerPreset();
					 
					\wpsg\wpsg_calculation::getSessionCalculation()->update();
					
					$bOK = $this->shop->basket->checkCheckout(2);
					
					$target_error = $this->shop->getURL(wpsg_ShopController::URL_CHECKOUT2);
					$target = $this->shop->getURL(wpsg_ShopController::URL_OVERVIEW);
					
				} else {
					
					die(__('Unerwarteter Fehler!', 'wpsg'));
					
				}
				
				// Speziell für PayPal PLUS, da an den IFrame gesendet wird
				if (wpsg_getStr($_REQUEST['wpsg_checkout']) === 'ppp') {
					
					unset($_SESSION['wpsg']['checkout']['payment']);
					
					if (!$bOK) die('<script> parent.wpsg_ppp_handleError(); </script>');
					else die('<script> parent.wpsg_ppp_run(); </script>');
					
				}
				
				if (!$bOK) {
					
					// Es sind Fehler aufgetreten, wieder zum Checkout leiten
					$this->redirect($target_error);
					
				} else {
					
					$this->redirect($target);
					
				}
				
				die();
				
			}
			
		} // public function template_redirect()
		
	} // class wpsg_BasketController extends wpsg_SystemController

