<?php

	/**
	 * Oberklasse aller wpsg Modelle
	 * @author Daschmi
	 */
	class wpsg_model {
		
		/** @var array */
		public static $_objCache = array();

		private $arMeta = null;

		/** @var wpsg_db */
		var $db = false;

		/** @var wpsg_ShopController */
		var $shop = false;
		
		var $id = false;		
		var $data = false;
		
		/**
		 * Konstruktor
		 */
		public function __construct()
		{
			
			$this->shop = $GLOBALS['wpsg_sc'];
			$this->db = $GLOBALS['wpsg_db'];
			
			$this->data = array();
			
		} // public function __construct()
		
		public function getId() {

		    $id = 0;

            if (isset($this->data['id'])) $id = intval($this->data['id']);

            return $id;
			
		}
		
		/**
		 * Lädt Daten des Objekts
		 */
		public function load($id)
		{
			
			$this->id = $id;
			
		} // public function load($id)

		/**
		 * Prüft ob das Objekt korrekt geladen wurde
		 * Wenn ein Datensatz in der DB nicht existiert, wird ein leeres Dummy Objekt geladen
		 */
		public function isLoaded() 
		{
			
			return wpsg_isSizedInt($this->data['id']);
			
		} // public function isLoaded() 
		
		/**
		 * Etwas im Datenarray ablegen
		 */
		public function __get($name) {

			if (array_key_exists($name, (array)$this->data)) {
				
				return $this->data[$name];
				
			}
			
			return null;
			
		} // public function __get($name)
		 
		
		/**
		 * Speichert etwas in einen Datenarry
		 */
		public function __set($name, $value)
		{
		
			$this->data[$name] = $value;
			 
		} // public function __set($name, $value)
		
		/**
		 * Gibt eine Instanz dieses Models zurück anhand des Primärschlüssels
		 * @param int $id
		 * @param bool $noCache
		 * @return static
		 */
		public static function getInstance($id, $noCache = false) {
						
			$class_name = get_called_class();
			
			if (wpsg_isSizedArray($id)) {
				
				$arReturn = [];
				
				foreach ($id as $_id) {
					
					$arReturn[$_id] = self::getInstance($_id);
					
				}
				
				return $arReturn;
				
			} else if (is_array($id)) {
				
				return [];
				
			} else {
				
				if (!isset(self::$_objCache[$class_name]) || !array_key_exists($id, self::$_objCache[$class_name]) || $noCache === true) {
										
					$oObject = new $class_name();
					$oObject->load($id);
					
					self::$_objCache[$class_name][$id] = $oObject;
					
				}
				
				return self::$_objCache[$class_name][$id];
				
			}
			
		} // public abstract static function getInstance($id)
		
		public static function clearCache($id = false) {
			
			$class_name = get_called_class();
			
			if ($id === false) {
				
				unset(self::$_objCache[$class_name]);
				
				self::$_objCache[$class_name] = [];
				
			}
			else {
				
				unset(self::$_objCache[$class_name][$id]);
				
			}
			
		}

        /**
         * Setzt einen Wert in der META Tabelle
         * @param $meta_key
         * @param null $meta_value
         * @throws \wpsg\Exception
         */
        public function setMeta($meta_key, $meta_value = null) {

            $class = get_called_class();

            if ($class::$_tableName === null) throw \wpsg\Exception::getInvalidValueException();
            if (!wpsg_isSizedInt($this->id)) throw \wpsg\Exception::getInvalidValueException();

            $target_id = intval($this->id);

            if ($target_id <= 0) throw \wpsg\Exception::getInvalidValueException();

            if (is_null($meta_value)) {

                $this->db->Query("DELETE FROM `".WPSG_TBL_META."` WHERE `meta_key` = '".wpsg_q($meta_key)."' AND `target_id` = '".wpsg_q($this->id)."' AND `meta_table` = '".wpsg_q($class::$_tableName)."' ");

            } else {

                $nExistsID = $this->db->fetchOne("SELECT M.`id` FROM `".WPSG_TBL_META."` AS M WHERE M.`meta_table` = '".wpsg_q($class::$_tableName)."' AND M.`meta_key` = '".wpsg_q($meta_key)."' AND `target_id` = '".wpsg_q($this->id)."' ");

                if (wpsg_isSizedInt($nExistsID)) {

                    $this->db->UpdateQuery(WPSG_TBL_META, array(
                        'meta_value' => wpsg_q($meta_value)
                    ), " `id` = '".wpsg_q($nExistsID)."' ");

                } else {

                    $this->db->ImportQuery(WPSG_TBL_META, array(
                        'target_id' => wpsg_q($this->id),
                        'meta_table' => wpsg_q($class::$_tableName),
                        'meta_key' => wpsg_q($meta_key),
                        'meta_value' => wpsg_q($meta_value)
                    ));

                }
				
				if ($this->arMeta !== null) $this->arMeta[$meta_key] = $meta_value;

            }

            $this->arMeta = null;

        } // public function setMeta($meta_key, $meta_value)

        /**
         * Gibt einen META Value zurück
         */
        public function getMeta($meta_key, $reload = false, $default = null) {

            $class = get_called_class();

            if ($class::$_tableName === null) throw \wpsg\Exception::getInvalidValueException();

            if ($this->arMeta === null || $reload === true) {

                $this->arMeta = $this->db->fetchAssocField("
                    SELECT
                        M.`meta_key`, M.`meta_value`
                    FROM 
                        `".WPSG_TBL_META."` AS M
                    WHERE
                        M.`meta_table` = '".wpsg_q($class::$_tableName)."' AND
                        M.`target_id` = '".wpsg_q($this->getId())."' 
                ", "meta_key", "meta_value");

            }

            if (array_key_exists($meta_key, $this->arMeta)) return $this->arMeta[$meta_key];
            else {
                
                if ($default !== null) return $default;
                else return '';
                
            }

        } // public function getMeta($meta_key)

	} // class wpsg_model
