<?php

	/**
	 * Gutscheinmodel
	 * @author Daschmi
	 */
	class wpsg_voucher extends wpsg_model {

		/**
		 * Lädt die Daten des Gutscheins
		 */
		public function load($voucher_id) {

			parent::load($voucher_id);

			$this->data = $this->db->fetchRow("
				SELECT 
					V.* 
				FROM
					`".WPSG_TBL_GUTSCHEIN."` AS V
				WHERE
					V.`id` = '".wpsg_q($voucher_id)."'
			");

			if (!wpsg_isSizedArray($this->data) || $this->data['id'] != $voucher_id || !wpsg_isSizedInt($voucher_id)) return false;

			return true;

		} // public function __construct($voucher_id)
				
		public function getFreeAmount() {
			
			if ($this->isMultiUsable() && !$this->isCoupon()) return $this->value;
			
			$free = wpsg_tf($this->value) - $this->getUsedAmount();
			
			if ($free < 0) $free = 0;
			
			return $free;
			
		}
		
		public function isMultiUsable() {

		    $multi = $this->multi;

			return wpsg_isSizedInt($multi);
			
		}
		
        public function getUsed() {

			return intval($this->db->fetchOne("SELECT COUNT(*) FROM `".WPSG_TBL_ORDER_VOUCHER."` WHERE `voucher_id` = '".wpsg_q($this->id)."' "));
			
        }
		
		public function getValue() {
			
			return wpsg_tf($this->value);
			
		}
        
        public function getUsedAmount($taxView = false) {
	
			if ($taxView === false) $taxView = $this->shop->getBackendTaxview();

			$strQueryWHERE = "";

			if (isset($_SESSION['wpsg']['order_id'])) {

				$strQueryWHERE .= " AND `order_id` != '".wpsg_q($_SESSION['wpsg']['order_id'])."' ";

			}

			if ($taxView === WPSG_NETTO) {
				
				$r = abs(wpsg_tf($this->db->fetchOne("SELECT SUM(`sum_netto`) FROM `".WPSG_TBL_ORDER_VOUCHER."` WHERE `voucher_id` = '".wpsg_q($this->getId())."' ".$strQueryWHERE)));
				
			} else if ($taxView === WPSG_BRUTTO) {
				
				$r = abs(wpsg_tf($this->db->fetchOne("SELECT SUM(`sum_brutto`) FROM `".WPSG_TBL_ORDER_VOUCHER."` WHERE `voucher_id` = '".wpsg_q($this->getId())."' ".$strQueryWHERE)));
				
			} else throw new \Exception(__('Nicht definiert.', 'wpsg'));
			
			$r = \apply_filters('wpsg_voucher_getUsedAmount', $r, $this->getId(), $taxView);
			
			return $r;
			
		}
        
        public function isCoupon() {
			
			return intval($this->__get('coupon')) > 0;
			
		}

        public function isAutoCreated() {

            $autocreate_order = $this->autocreate_order;
                
            return wpsg_isSizedInt($autocreate_order);
		    
        }

        public function getAllowedProductgroup(): array {

			return wpsg_explode(',', $this->__get('productgroups'));

        }

        public function isLimitedProductgroup(): bool {

			$arAllowedProductGroups = wpsg_explode(',', $this->__get('productgroups'));

			return wpsg_isSizedArray($arAllowedProductGroups);

        }

        public function getAllowedProduct(): array {

			return wpsg_explode(',', $this->__get('products'));

        }

        public function isLimitedProduct(): bool {

			$arAllowedProductGroups = wpsg_explode(',', $this->__get('products'));

			return wpsg_isSizedArray($arAllowedProductGroups);

        }

		/**
		 * Prüft ob der Gutschein/Coupon anwendbar ist 
		 * @param null $product_id
		 * @param bool $bCheckAdd
		 *
		 * @return bool
		 */
        public function isUsabel($product_id = null, $bCheckAdd = true) {
			
        	if ($bCheckAdd) {
			
        		if (strtotime($this->start_date) > time()) return false;
				if (strtotime($this->end_date) < time()) return false;
				if (!$this->isMultiUsable() && $this->getUsed() > 0) return false;
				if ($this->getFreeAmount() <= 0) return false;
				
	        }

			if ($product_id !== null) {

				if ($this->isLimitedProduct()) {

					if (!in_array($product_id, $this->getAllowedProduct())) return false;

				}

				if ($this->isLimitedProductgroup()) {

					if (!in_array(wpsg_product::getInstance($product_id)->getProductgroup()->getId(), $this->getAllowedProductgroup())) return false;

				}

			}

			return true;
			
		}
        
        public function getStatusLabel()
        {
        	
        	if ($this->getFreeAmount() <= 0) return __('Verbraucht');

			if ($this->multi === '1')
			{
				
				if (current_time('timestamp') < strtotime($this->start_date)) return __('Wartend', 'wpsg');
				
				if (current_time('timestamp') < strtotime($this->start_date) || current_time('timestamp') > strtotime($this->end_date))
				{
					
					if ($this->getUsed() > 0) return __('Verbraucht', 'wpsg');
					else return __('Inaktiv', 'wpsg');
					
				}
				else return __('Aktiv', 'wpsg');
				
			}
			else
			{
				
				if (current_time('timestamp') < strtotime($this->start_date)) return __('Wartend', 'wpsg');
				
				if ($this->getUsed() > 0) return __('Verbraucht', 'wpsg');
				else
				{
				
					if (current_time('timestamp') < strtotime($this->start_date) || current_time('timestamp') > strtotime($this->end_date)) return __('Inaktiv', 'wpsg');
					else return __('Aktiv', 'wpsg');
					
				}
				
			}

        }

		/**
		 * 0	// Alle
	     * 1	// Aktiv
		 * 2	// Inaktiv
		 * 3	// Verbraucht
		 * 4	// Wartend
		 * 
		 * @return int
		 */
        public function getStatus() {
        	
        	if ($this->__get('multi') === '1') {
        		
        		if (current_time('timestamp') < strtotime($this->start_date)) return 4;	// Wartend
        		
        		if (current_time('timestamp') < strtotime($this->start_date) || current_time('timestamp') > strtotime($this->end_date)) {
        			
        			if ($this->getUsed() > 0) return 3;   //('Verbraucht', 'wpsg');
        			else return 2; // Inaktiv
        			
        		} else return 1; // Aktiv
        		
        	} else {
        		
        		if ($this->getUsed() > 0) return 3;	// Verbraucht
        		else {
					
        			if (current_time('timestamp') < strtotime($this->start_date) || current_time('timestamp') > strtotime($this->end_date)) return 2; 	// Inaktiv
        			else return 1; 	// Aktiv
        			
        		}
        		
        	}
        	
        }
        
        public function delete()
        {

            $this->db->Query("DELETE FROM `".WPSG_TBL_GUTSCHEIN."` WHERE `id` = '".wpsg_q($this->id)."' ");

        }

        /* Statische Funktionen */

		public static function count($arFilter = array()) {

			return sizeof(self::find($arFilter));
			 
		} // public static function count($arFilter = array())

		/**
		 * Gibt einen Array von Kundengruppen zurück, die auf den übergebenen Filter passen
		 * @param array $arFilter
		 */
		public static function find($arFilter = array()) {

			list($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);

			$strQuery = "
				SELECT
					V.`id`
					".$strQuerySELECT."
				FROM
					`".WPSG_TBL_GUTSCHEIN."` AS V
						".$strQueryJOIN."
				WHERE
					1
					".$strQueryWHERE."
				HAVING
					1
					".$strQueryHAVING."
				ORDER BY
					".$strQueryORDER."						
			";
			
			$arID = $GLOBALS['wpsg_db']->fetchAssocField($strQuery);
			$arReturn = array();

			$cnt = array();
			 
			$cnt[0] = 0;	// Alle
			$cnt[1] = 0;	// Aktiv
			$cnt[2] = 0;	// Inaktiv
			$cnt[3] = 0;	// Verbraucht
			$cnt[4] = 0;	// Wartend
			 
			foreach ($arID as $id) {

				$oVoucher = self::getInstance($id);
				$state = intval($oVoucher->getStatus());
				
				$cnt[0] ++;
				
				$cnt[$state] ++;
				
				if (intval($arFilter['status']??0) > 0) {
					
					if ($state === intval($arFilter['status'])) {
						
						$arReturn[$id] = $oVoucher;
						
					}
					
				} else $arReturn[$id] = $oVoucher;
				 				
			}

			if (isset($arFilter['limit'])) {
				
				$arReturn = array_slice($arReturn, $arFilter['limit'][0], $arFilter['limit'][1]);
				
			}
			
			$arReturn['counts'] = $cnt;
			
			return $arReturn;


		} // public static function find($arFilter = array())

		public static function getQueryParts($arFilter = array()) {

			$strQuerySELECT = "";
			$strQueryWHERE = "";
			$strQueryJOIN = "";
			$strQueryHAVING = "";
			$strQueryORDER = "";

 			if (wpsg_isSizedString($arFilter['s'])) $strQueryWHERE .= " AND V.`code` LIKE '%".wpsg_q($arFilter['s'])."%' ";

			if (wpsg_isSizedString($arFilter['code'])) $strQueryWHERE .= " AND V.`code` LIKE '%".wpsg_q($arFilter['code'])."%' ";

			if (wpsg_isSizedString($arFilter['order'], 'code')) { $strQueryORDER = " V.`code` "; }
            else if (wpsg_isSizedString($arFilter['order'], 'start_date')) { $strQueryORDER = " V.`start_date` "; }
            else if (wpsg_isSizedString($arFilter['order'], 'end_date')) { $strQueryORDER = " V.`end_date` "; }
            else if (wpsg_isSizedString($arFilter['order'], 'value')) { $strQueryORDER = " V.`value` "; }
			else $strQueryORDER = " V.`id` ";

			// Richtung
			if (wpsg_isSizedString($arFilter['ascdesc'], "DESC")) $strQueryORDER .= " DESC ";
			else $strQueryORDER .= " ASC ";

			return array($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER);

		}

        /**
         * @param int $voucher_id
         * @param bool $noCache
         * @return wpsg_voucher
         */
		public static function getInstance($voucher_id, $noCache = false) {

			$class_name = get_called_class();

			if (wpsg_isSizedArray($voucher_id)) {

				$arReturn = [];

				foreach ($voucher_id as $_id) {

					$arReturn[$_id] = self::getInstance($_id);

				}

				return $arReturn;

			} else if (is_array($voucher_id)) {

			    return array();

            } else {

				if (!array_key_exists($class_name.'_'.$voucher_id, self::$_objCache) || $noCache === true) {

					$oObject = new $class_name();
					$oObject->load($voucher_id);

					self::$_objCache[$class_name.'_'.$voucher_id] = $oObject;

				}

				return self::$_objCache[$class_name.'_'.$voucher_id];

			}

		} // public static function getInstance($id, $noCache = false)

	} // class wpsg_voucher
