<?php

	/**
	 * Klasse, die verschiedene Zahlungsarten über den micropayment.de Dienstleister ermöglicht
	 * Ist nur für die Lizenztypen Pro und Enterpreise zulässig
	 * @author Daschmi (05.07.2013)
	 */
	class wpsg_mod_micropayment_paywindow extends wpsg_mod_basic {
		
		var $lizenz = 2;
		var $id = 1750;
		var $hilfeURL = 'http://wpshopgermany.de/?p=3424';

		var $arPaymentWindow = [];
		var $default_pay_title = 'O%order_onr% K%kunde_knr%';
		var $default_pay_text = 'Bezahlung zur Bestellung %order_onr%';

		/**
		 * Costructor
		 */
		public function __construct() {
				
			parent::__construct();
				
			$this->name = __('micropayment™', 'wpsg');
			$this->group = __('Zahlungsarten', 'wpsg');
			$this->desc = __('Ermöglicht die Zahlung über den Zahlungsdienstleister <a href="http://r132.micropayment.de">micropayment™</a>.', 'wpsg');

            $this->arPaymentWindow = [
                1 => [
                    'code' => 'creditcard',
                    'title' => 'Kreditkarte',
                    'url' => 'https://creditcard.micropayment.de/creditcard/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/creditcard.png')
                ],
                2 => [
                    'index' => 'lastschrift',
                    'title' => 'Lastschrift',
                    'url' => 'https://sepadirectdebit.micropayment.de/lastschrift/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/lastschrift.png')
                ],
                3 => [
                    'index' => 'sofort',
                    'title' => 'Sofort.',
                    'url' => 'https://directbanking.micropayment.de/sofort/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/sofort.png')
                ],
                4 => [
                    'index' => 'prepay',
                    'title' => 'Vorkasse',
                    'url' => 'https://prepayment.micropayment.de/prepay/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/prepay.png')
                ],
                5 => [
                    'index' => 'paypal',
                    'title' => 'PayPal',
                    'url' => 'https://paypal.micropayment.de/paypal/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/paypal.png')
                ],
                6 => [
                    'index' => 'paydirekt',
                    'title' => 'paydirekt',
                    'url' => 'https://paydirekt.micropayment.de/paydirekt/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/paydirekt.png')
                ],
                7 => [
                    'index' => 'paysafecard',
                    'title' => 'paysafecard',
                    'url' => 'https://paysafecard.micropayment.de/paysafecard/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/paysafecard.png')
                ],
                8 => [
                    'index' => 'call2pay',
                    'title' => 'Call2Pay',
                    'url' => 'https://call2pay.micropayment.de/call2pay/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/call2pay.png')
                ],
                9 => [
                    'index' => 'handypay',
                    'title' => 'HandyPay',
                    'url' => 'https://mobilepayment.micropayment.de/handypay/event/',
                    'logo' => $this->shop->getRessourceURL('mods/mod_micropayment_paywindow/gfx/handypay.png')
                ]
            ];
			
		} // public function __construct()

        public function install() {

            $this->shop->checkDefault('wpsg_mod_micropayment_autorun', '1');

        }

		public function settings_edit() {
			
			$pages = get_pages();
			
			$arPages = array(
				'-1' => __('Neu anlegen und zuordnen', 'wpsg')
			);
			
			foreach ($pages as $k => $v) {

				$arPages[$v->ID] = $v->post_title.' (ID:'.$v->ID.')';

			}

            $view = [];
			$view['arPaymentWindow'] = $this->arPaymentWindow;
            $view['arPages'] = $arPages;
            $view['notification_url'] = $this->shop->getURL(wpsg_ShopController::URL_BASKET);
            $view['notification_password'] = $this->getNotificationPassword();
            $view['default_pay_title'] = $this->default_pay_title;
            $view['default_pay_text'] = $this->default_pay_text;

            $this->shop->view['wpsg_mod_micropayment_paywindow'] = $view;
			$this->shop->render(WPSG_PATH_VIEW.'/mods/mod_micropayment_paywindow/settings_edit.phtml');
			
		} // public function settings_edit()
		
		public function settings_save() {

            $this->shop->update_option('wpsg_mod_micropayment_paywindow_accesskey', $_REQUEST['wpsg_mod_micropayment_paywindow_accesskey'], false, false, WPSG_SANITIZE_APIKEY);
            $this->shop->update_option('wpsg_mod_micropayment_paywindow_projectid', $_REQUEST['wpsg_mod_micropayment_paywindow_projectid'], false, false, WPSG_SANITIZE_TEXTFIELD);
            $this->shop->createPage(__('Fehlgeschlagene Micropayment Zahlung', 'wpsg'), 'wpsg_mod_micropayment_paywindow_page_error', intval($_REQUEST['wpsg_mod_micropayment_paywindow_page_error']));
            $this->shop->update_option('wpsg_mod_micropayment_paywindow_paytitle', $_REQUEST['wpsg_mod_micropayment_paywindow_paytitle'], false, false, WPSG_SANITIZE_TEXTFIELD);
            $this->shop->update_option('wpsg_mod_micropayment_paywindow_paytext', $_REQUEST['wpsg_mod_micropayment_paywindow_paytext'], false, false, WPSG_SANITIZE_TEXTFIELD);
            $this->shop->update_option('wpsg_mod_micropayment_paywindow_sandbox', $_REQUEST['wpsg_mod_micropayment_paywindow_sandbox'], false, false, WPSG_SANITIZE_TEXTFIELD);
            $this->shop->update_option('wpsg_mod_micropayment_autorun', $_REQUEST['wpsg_mod_micropayment_autorun'], false, false, WPSG_SANITIZE_CHECKBOX);

            foreach ($this->arPaymentWindow as $pw_key => $pw) {

                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_activ', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_activ'], false, WPSG_SANITIZE_CHECKBOX);
                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_title', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_title'], false, WPSG_SANITIZE_TEXTAREA);
                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_hint', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_hint'], false, WPSG_SANITIZE_TEXTAREA);
                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_logo', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_logo'], false, WPSG_SANITIZE_URL);
                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_gebuehr', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_gebuehr'], false, WPSG_SANITIZE_FLOAT);
                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_mwst', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_mwst'], false, WPSG_SANITIZE_TAXKEY);
                $this->shop->update_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_mwstland', $_REQUEST['wpsg_mod_micropayment_paywindow_'.$pw_key.'_mwstland'], false, WPSG_SANITIZE_CHECKBOX);

            }

		} // public function settings_save()

		public function addPayment(&$arPayment) {

		    foreach ($this->arPaymentWindow as $pw_key => $pw) {

		        $active = intval($this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_activ'));
                $price = wpsg_ff($this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_gebuehr'));
                $tax_key = $this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_mwst');
                $tax_null = $this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_mwstland');
                $hint = __(\sanitize_text_field($this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_hint')));

                if (!wpsg_isSizedString($logo)) $logo = $pw['logo'];

		        if ($active === 1) {

		            $payment_code = $this->id.'_'.$pw_key;

                    $arPayment[$payment_code] = array(
                        'id' => $payment_code,
                        'name' => $this->getTitle($payment_code),
                        'price' => $price,
                        'tax_key' => $tax_key,
                        'mwst_null' => $tax_null,
                        'hint' => $hint,
                        'logo' => $this->getLogo($payment_code)
                    );

                }

            }

		} // public function addPayment(&$arPayment)
		 
		public function order_done(&$order_id, &$done_view) {
			
			$oOrder = wpsg_order::getInstance($order_id);

			// Bestellungen mit 0 geben nix aus
			if ($oOrder->getToPay() <= 0) return;

            $payment_code = $this->shop->view['basket']['checkout']['payment'];

			if (preg_match('/^'.$this->id.'_\d+$/', $payment_code)) {

                $view = [
                    'order_status' => $oOrder->getStatusCode(),
				    'logo' => $this->getLogo($payment_code),
                    'url' => $this->getUrl($payment_code, [
                        'amount' => $oOrder->getToPay(WPSG_BRUTTO) * 100,
                        'currency' => $this->shop->get_option('wpsg_currency'),
                        'title' => $this->getPayTitle($oOrder),
                        'paytext' => $this->getPayText($oOrder),
                        'testmode' => (($this->shop->get_option('wpsg_mod_micropayment_paywindow_sandbox') === '1')?'1':'0'),
                        'userid ' => $oOrder->getCustomer()->getId(),
                        'orderid' => $oOrder->getId()
                    ]),
                    'title' => $this->getTitle($payment_code)
                ];

                $this->shop->view['wpsg_mod_micropayment_paywindow'] = $view;
				$this->shop->render(WPSG_PATH_VIEW.'/mods/mod_micropayment_paywindow/order_done.phtml');
				
			}
		
		} // public function order_done(&$order_id)
		 
		public function mail_payment() {
		
			if (!preg_match('/^'.$this->id.'_\d+$/', $this->shop->view['basket']['checkout']['payment'])) return;
			
			if ($this->shop->htmlMail === true) {
				
				echo '<a href="'.$this->shop->getDoneURL($this->shop->view['o_id']).'">'.__('Zahlungslink', 'wpsg').'</a>'.__(', um die Zahlung durchzuführen', 'wpsg');						
				
			} else {
			
				echo wpsg_pad_right(__('Zahlungslink', 'wpsg').':', 35).$this->shop->getDoneURL($this->shop->view['o_id']);
				
			}
				
		} // public function mail_payment()
		
		public function template_redirect() {

            if (\get_the_ID() === intval($this->shop->get_option('wpsg_page_basket')) && wpsg_isSizedString($_REQUEST['function'], 'billing')) {

                $oOrder = wpsg_order::getInstance($_REQUEST['order_id']);

                if ($oOrder->isLoaded()) {

                    // Ich ignoriere Zahlungen die bereits extern gezahlt gemeldet wurden
                    //if ($oOrder->__get('ext_payed') === '1') exit;

                    try {

                        // DNS Backcheck
                        $arHosts = ['proxy.micropayment.de', 'access.micropayment.de', 'service.micropayment.de', 'micropayment.de'];
                        $host = gethostbyaddr($_SERVER['REMOTE_ADDR']);

                        if (!in_array($host, $arHosts)) {

                            throw new \Exception(
                                wpsg_translate(__('Hostname #1# ist nicht in den erlaubten Hosts (#2#) enthalten', 'wpsg'), $host, implode(', ', $arHosts))
                            );

                        }

                        if ($this->getNotificationPassword() !== wpsg_getStr($_REQUEST['notification_password'])) {

                            throw new \Exception(
                                wpsg_translate(
                                    __('Übermitteltes Benachrichtigungspasswort (#1#) stimmt nicht mit dem hinterlegten (#2#) überein.', 'wpsg'),
                                    wpsg_getStr($_REQUEST['notification_password']),
                                    $this->getNotificationPassword()
                                )
                            );

                        }

                        if ($oOrder->getToPay(WPSG_BRUTTO) !== wpsg_getFloat($_REQUEST['amount']) / 100) {

                            throw new \Exception(
                                wpsg_translate(
                                    __('Übermittelter Zahlbetrag (#1#) stimmt nicht mit der Bestellung (#2#) überein.', 'wspg'),
                                    wpsg_getFloat($_REQUEST['amount']) / 100,
                                    $oOrder->getToPay(WPSG_BRUTTO)
                                )
                            );

                        }


                        if ($this->shop->setPayMent($oOrder->getId(), $_REQUEST['amount'] / 100)) {

                            $this->shop->setOrderStatus($oOrder->getId(), 100, true);

                            $this->db->UpdateQuery(WPSG_TBL_ORDER, ['ext_payed' => '1'], " `id` = '".wpsg_q($oOrder->getId())."' ");

                            $oOrder->addLogEntry(__('MicroPayment Zahlungseingang', 'wpsg'), print_r($_REQUEST, true));

                        } else throw new \Exception(__('Zahlung konnte nicht validiert werden.', 'wpsg'));

                        $url = wpsg_ShopController::getShop()->getDoneURL($oOrder->getId());

                    } catch (\Exception $e) {

                        $oOrder->addLogEntry(__('MicroPayment Fehler', 'wpsg'), $e->getMessage()."\r\n".print_r($_REQUEST, true));
                        $url = \get_permalink($this->shop->get_option('wpsg_mod_micropayment_paywindow_page_error'));

                    }

                } else {

                    $url = \get_permalink($this->shop->get_option('wpsg_mod_micropayment_paywindow_page_error'));

                }

                $delimiter = "\n";
                $status = 'ok';
                $target = '_self';
                $forward = '1';

                $response = '';
                $response .= 'status='.$status;
                $response .= $delimiter;
                $response .= 'url='.$url;
                $response .= $delimiter;
                $response .= 'forward='.$forward;
                $response .= $delimiter;
                $response .= 'target='.$target;
                $response .= $delimiter;

                echo $response;
                exit;

            }
				
		} // public function template_redirect()			

		/** Modulfunktionen */

        /**
         * @param $payment_code z.B. 1750_0
         * @return string
         * @throws Exception
         */
        public function getTitle($payment_code) {

            if (!preg_match('/^'.$this->id.'_\d+$/', $payment_code)) throw \wpsg\Exception::getInvalidValueException();

            $pw_key = explode('_', $payment_code)[1];

            $title = \sanitize_text_field($this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_title'));
            if (!wpsg_isSizedString($title)) $title = $this->arPaymentWindow[$pw_key]['title'];

            return $title;

        }

        /**
         * @param $payment_code z.B. 1750_0
         * @return string
         * @throws Exception
         */
        public function getUrl($payment_code, $arData) {

            if (!preg_match('/^'.$this->id.'_\d+$/', $payment_code)) throw \wpsg\Exception::getInvalidValueException();

            $pw_key = explode('_', $payment_code)[1];

            $strGETQuery = http_build_query($arData + [
                'project' => $this->shop->get_option('wpsg_mod_micropayment_paywindow_projectid')
            ]);

            $url = $this->arPaymentWindow[$pw_key]['url'].'?'.$strGETQuery;
            $url = $this->sealUrl($url);

            return $url;

        }

        /**
         * @param $payment_code z.B. 1750_0
         * @return string
         * @throws Exception
         */
        public function getLogo($payment_code) {

		    if (!preg_match('/^'.$this->id.'_\d+$/', $payment_code)) throw \wpsg\Exception::getInvalidValueException();

		    $pw_key = explode('_', $payment_code)[1];

            $logo = __(\esc_url_raw($this->shop->get_option('wpsg_mod_micropayment_paywindow_'.$pw_key.'_logo')));
            if (!wpsg_isSizedString($logo)) $logo = $this->arPaymentWindow[$pw_key]['logo'];

            return $logo;

        }

        public function getPayTitle($oOrder) {

            $pay_title = $this->shop->get_option('wpsg_mod_micropayment_paywindow_paytitle');

            if (!wpsg_isSizedString($pay_title)) $pay_title = $this->default_pay_title;

            $pay_title = $this->shop->replaceUniversalPlatzhalter($pay_title, $oOrder->getId());

            return $pay_title;

        }

        public function getPayText($oOrder) {

            $pay_text = $this->shop->get_option('wpsg_mod_micropayment_paywindow_paytext');

            if (!wpsg_isSizedString($pay_text)) $pay_text = $this->default_pay_text;

            $pay_text = $this->shop->replaceUniversalPlatzhalter($pay_text, $oOrder->getId());

            return $pay_text;

        }

        /**
         * Versiegelt die URL
         * https://www.micropayment.de/help/integration_web/?reset=1&context=504a5574356e6e726f6335424d47306c4a517134715237654a374c6c6c4a4f35375549336f687943306e4d3d&_r=mbr&_src=ctor
         *
         * @param $url
         * @return string
         */
        private function sealUrl($url) {

            $accessKey = $this->shop->get_option('wpsg_mod_micropayment_paywindow_accesskey');

            $matches = [];
            preg_match('/^(http(?:s?):\/\/[^?]*?\?)?\?*(.*?)(?:&?seal=([^&]*)(&?.*)?)?$/', $url, $matches);
            $matches = array_merge($matches, [1 => '', 2 => '', 3 => '', 4 => '']);

            return (string)$matches[1].(string)$matches[2].'&seal='.md5(urldecode((string)$matches[2]).(string)$accessKey).(string)$matches[4];

        }

        public function getNotificationPassword() {

            $notification_password = $this->shop->get_option('wpsg_mod_micropayment_paywindow_notification_password');

            if (!wpsg_isSizedString($notification_password)) {

                $notification_password = wpsg_genCode(32);

                $this->shop->update_option('wpsg_mod_micropayment_paywindow_notification_password', $notification_password);

            }

            return $notification_password;

        }

	} // class wpsg_mod_micropayment extends wpsg_mod_basic

